@@ -4,7 +4,4 @@
     :license: GNU GPL v3 or above, see LICENSE for more details.
 """
 
-__version__ = "0.11.0"
-
-# https://docs.djangoproject.com/en/2.0/ref/applications/#configuring-applications-ref
-default_app_config = "inventory.apps.InventoryConfig"
+__version__ = "0.12.0"
